; STORM (16K ZX81)
; by C Hornby
; from Sinclair User 26 (November 1983), pages 75-76
;
; Disassembly by Paul Farrow (1st October 2016)

; -----------------
; Program variables
; -----------------

4082 1C        DEFW    $1C1C			; Holds the position of the player within the
display file.
4084 1C        DEFB    $1C			; Holds the number of lives remaining.

4085 1C        DEFB    $1C, $1c, $1C		; Unused.

; ------------------------
; Entry Point For The Game (USR 16520)
; ------------------------

; Reset the displayed score.

4088 2A0C40    LD      HL,($400C)		; Fetch the address of the display file.
408b 23        INC     HL			; Skip over the initial NEWLINE character.

408c 0608      LD      B,$08			; There are 8 digits in the displayed score.

408e 361C      LD      (HL),$1C			; Reset a digit of the score to '0'.
4090 23        INC     HL			; Advance to the next display file position.
4091 05        DEC     B			; Decrement the score digit counter.
4092 20FA      JR      NZ,$408E			; Repeat for all digits of the score.

; Reset the displayed number of lives.

4094 23        INC     HL			; Advance to the display file position showing
the number of lives.
4095 0603      LD      B,$03			; The game starts with 3 lives.

4097 363B      LD      (HL),$3B			; Display a 'V' denoting a life.
4099 23        INC     HL			; Advance to the next display file position.
409a 05        DEC     B			; Decrement the life indication counter.
409b 20FA      JR      NZ,$4097			; Repeat for all lives.

409d 2A0C40    LD      HL,($400C)		; Fetch the address of the display file.
40a0 112300    LD      DE,$0023			; Offset to the row 1 column 1.
40a3 19        ADD     HL,DE
40a4 228240    LD      ($4082),HL		; Store as the initial starting position
of the player.

40a7 3E04      LD      A,$04			; The number of lives + 1. This will be
decremented below before the game starts.
40a9 328440    LD      ($4084),A		; Store the lives counter.

; Joins here each time the player is hit.

40ac 3A8440    LD      A,($4084)		; Fetch the lives counter.
40af 3D        DEC     A			; Decrement the number of lives
40b0 2001      JR      NZ,$40B3			; Jump ahead if lives remains.

40b2 C9        RET     				; All lives lost so return to BASIC.

40b3 328440    LD      ($4084),A		; Store the new number of lives remaining.

; Erase the current life from the number of lives remaining indicators.

40b6 2A0C40    LD      HL,($400C)		; Fetch the address of the display file.
40b9 110900    LD      DE,$0009			; Offset to row 0 column 8, the position
before the first displayed lives remaining indicator.
40bc 19        ADD     HL,DE
40bd 5F        LD      E,A			; Offset by the number of lives remaining, i.e.
point at the right most remaining indicator.
40be 19        ADD     HL,DE
40bf 3600      LD      (HL),$00			; Erase the life indicator.

; --------------
; Main Game Loop
; --------------

; Increment the score.

40c1 2A0C40    LD      HL,($400C)		; Fetch the address of the display file.
40c4 110800    LD      DE,$0008			; Offset to row 0 column 7, the right most
digit of the displayed score.
40c7 19        ADD     HL,DE

40c8 34        INC     (HL)			; Increment the value of the digit.

40c9 0608      LD      B,$08			; There are 8 digits forming the score.

40cb 7E        LD      A,(HL)			; Fetch the new value of the digit.
40cc FE26      CP      $26			; Has it increment above '9', i.e. to 'A'?
40ce 2005      JR      NZ,$40D5			; Jump ahead if not.

40d0 361C      LD      (HL),$1C			; Reset the digit to '0'.
40d2 2B        DEC     HL			; Point to the next significant digit of the
score.
40d3 34        INC     (HL)			; Increment the digit.
40d4 23        INC     HL			; Point back to the previous digit of the score
(will be reversed below).

40d5 2B        DEC     HL			; Point to the next significant digit of the
score.
40d6 05        DEC     B			; Decrement the digit counter.
40d7 20F2      JR      NZ,$40CB			; Loop back to process all digits of the
score.

; Process movement of the player.

40d9 2A8240    LD      HL,($4082)		; Fetch the display file position of the
player.
40dc 3600      LD      (HL),$00			; Erase the player.

40de 3A2540    LD      A,($4025)		; Fetch the LAST_K system variable.
40e1 CB5F      BIT     3,A			; Has a key between 1 to 5 been pressed?
40e3 2001      JR      NZ,$40E6			; Jump ahead if not.

40e5 2B        DEC     HL			; Move the player to the left. [Bug in the
magazine listing had this as $2A]

40e6 CB67      BIT     4,A			; Has a key between 6 to 0 been pressed?
40e8 2001      JR      NZ,$40EB			; Jump ahead if not.

40ea 23        INC     HL			; Move the player to the right.

40eb 3E76      LD      A,$76			; The end of row NEWLINE character.
40ed BE        CP      (HL)			; Is the new position off the side screen?
40ee 2803      JR      Z,$40F3			; Jump ahead if it is.

40f0 228240    LD      ($4082),HL		; Save the new player position.

; Scroll the screen up.

40f3 2A0C40    LD      HL,($400C)		; Fetch the address of the display file.
40f6 112100    LD      DE,$0021			; Offset to row 1 column 0, ie. the scroll
'TO' position.
40f9 19        ADD     HL,DE
40fa E5        PUSH    HL			; Save the 'TO' position.

40fb 19        ADD     HL,DE			; Offset to row 2 column 0, i.e. the scroll
'FROM' position.
40fc D1        POP     DE			; Retrieve the 'TO' position.

40fd 01D602    LD      BC,$02D6			; There are 22 rows of 33 columns to be
shifted.
4100 EDB0      LDIR    				; Shift the playing area within the display file
up one row, which results in
						; row 22 being cleared with spaces since row 23 is always empty, i.e.
all spaces.

4102 2A8240    LD      HL,($4082)		; Fetch the display file position of the
player.
4105 3E80      LD      A,$80			; The character code of an inverse space,
i.e. an obstacle to avoid.
4107 BE        CP      (HL)			; Has the player been hit?
4108 CAAC40    JP      Z,$40AC			; Jump if so to decrement the lives
counter.

410b 363B      LD      (HL),$3B			; Insert the player 'V' into the display
file.

410d 0601      LD      B,$01			; Fetch the Difficulty Level, i.e. obstacle
count per row.

410f 2A0C40    LD      HL,($400C)		; Fetch the address of the display file.
4112 11D602    LD      DE,$02D6			; Offset to row 21 column 32, i.e. the
NEWLINE terminator for the row.
4115 19        ADD     HL,DE

; Insert obstacles on the new row at the bottom of the screen.

4116 ED5B3440  LD      DE,($4034)		; Fetch the FRAMES system variable.
411a CBBA      RES     7,D
411c CBB2      RES     6,D			; Mask off the upper 3 bits to yield a value
between $0000 and $1FFF.
411e CBAA      RES     5,D			; This is then used as a look up address within
the ROM, cleverly
4120 1A        LD      A,(DE)			; returning what will appear to be a random
number.

4121 CBBF      RES     7,A
4123 CBB7      RES     6,A			; Mask off the upper 3 bits of the value read
from ROM to reduce the
4125 CBAF      RES     5,A			; value to between $00 and $1F, i.e column
range 0 to 31.

4127 3C        INC     A			; Increment to form an offset with value between
$01 and $20.
4128 1600      LD      D,$00
412a 5F        LD      E,A			; Offset from the terminator character of row
21 to select a character
412b 19        ADD     HL,DE			; position within row 22.

412c 3680      LD      (HL),$80			; Insert an inverse space, i.e. an
obstacle to avoid.
412e 05        DEC     B			; Decrement the Difficulty Level, i.e. obstacle
count per row.
412f 20DE      JR      NZ,$410F			; Loop back for all obstacles to insert
into the row.

4131 3E04      LD      A,$04			; Fetch the Speed Level, i.e. outer delay
counter.

4133 0600      LD      B,$00			; Inner delay counter.

4135 05        DEC     B			; Decrement the inner delay counter.
4136 20FD      JR      NZ,$4135			; Repeat until it reaches 0.

4138 3D        DEC     A			; Decrement the outer delay counter.
4139 20F8      JR      NZ,$4133			; Repeat until it reaches 0.

413b C3C140    JP      $40C1			; Jump back to repeat the game loop.

